/*
 * © 2024 Browsolate Limited. All rights reserved.
 *
 * This source code is the property of Browsolate Limited and is strictly confidential.
 * Unauthorized copying, modification, distribution, or use of this code, in part or in whole,
 * without prior written permission from Browsolate Limited is strictly prohibited and may
 * result in legal action.
 *
 * For licensing inquiries, please contact Browsolate Limited at https://browsolate.com.
 *
 * Author: Browsolate Limited
 * Version: 1.0
 * Date: 2024-11-02
 */

import { encrypt, apiEncrypt } from './encryption.js';
// Fetch viewer_options.json from the specified host
export async function loadViewerOptions(b8Host) {
    const jsonUrl = `${b8Host}/viewer_options.json`;
    try {
        const response = await fetch(jsonUrl);
        if (!response.ok) {
            throw new Error(`Failed to load viewer options from ${jsonUrl}`);
        }

        const data = await response.json();
        if (!Array.isArray(data)) {
            throw new Error("Expected an array in viewer_options.json");
        }

        return data;
    } catch (error) {
        console.log(error);
    }
    return {};
}

export function isValidUrl(value) {
    try {
        new URL(value);
        return true;
    } catch {
        return false;
    }
}

export function getValidHttpUrl(urlString) {
    try {
        const parsedUrl = new URL(urlString);
        if(parsedUrl.protocol === "http:" || parsedUrl.protocol === "https:") {
            return parsedUrl;
        }
    } catch (error) {
        // If the URL is invalid, URL constructor will throw an error
        return null;
    }
    return null;
}

export function _matchPattern(url, patterns) {
    try {
        // Ensure patterns is an array
        if (!Array.isArray(patterns)) {
            return false;
        }

        // Extract the hostname from the given URL
        const hostname = url.hostname;
        const pathname = url.pathname;

        // Check if any domain is a suffix of the hostname
        return patterns.some((domain) => {
            if(domain === '*') {
                return true;
            } else if(!(domain.startsWith("https://") || domain.startsWith("http://"))) {
                domain = `http://${domain}`;
            }
            let patURL = new URL(domain);
            return hostname.endsWith(patURL.hostname) && pathname.startsWith(patURL.pathname);
        } );
    } catch (e) {
        console.error(`Error processing URL "${url}": ${e.message}`);
        return false; // Return false if there's an error
    }
}

export function getStoredValue(key) {
    return new Promise((resolve, reject) => {
        chrome.storage.sync.get(
            [key],
            (items) => {
                resolve(items[key]);
            }
        );
    });
}

export function setStoredValue(key, value) {
    return new Promise((resolve, reject) => {
        const data = {};
        data[key] = value;

        chrome.storage.sync.set(data, () => {
            if (chrome.runtime.lastError) {
                reject(chrome.runtime.lastError);
            } else {
                resolve();
            }
        });
    });
}


export function extractFilenameFromUrl(url) {
    try {
        const parsedUrl = new URL(url);

        // Extract the pathname (everything after the domain)
        const pathname = parsedUrl.pathname;

        // Get the last segment of the pathname as the filename
        const filename = pathname.split("/").pop();

        // Return null if the filename is empty (e.g., a trailing slash)
        return filename && filename.includes(".") ? filename : null;
    } catch (error) {
        console.error(`Invalid URL: ${url}`, error);
        return null;
    }
}

// Generate an encrypted URL with query parameters from stored viewerOptions
export async function generateEncryptedUrl(userUrl) {
    // Get b8Host from storage and append /launcher.html to create the base URL
    let b8Host = await getStoredValue("b8Host");
    if (!b8Host) {
        throw new Error("b8Host is not defined. Please enter a valid host.");
    }
    b8Host = b8Host.replace(/\/+$/, '');

    const baseUrl = `${b8Host}/viewer.html`;
    let url = new URL(baseUrl);
    let queryParams = {};

    // Add the provided user URL to the query parameters with the name 'url'
    if (!isValidUrl(userUrl)) {
        throw new Error("Invalid URL format for user URL.");
    }
    url.searchParams.set("url", userUrl);
    queryParams["url"] = userUrl;

    // Get viewerOptions from chrome.storage
    const storedOptions = await getStoredValue("viewerOptions") || [];

    // Process each option according to its type
    storedOptions.forEach(param => {
        const { name, value, type } = param;

        if (type === 'keyValue') {
            // Handle key-value pairs stored as comma-separated strings
            if (value) {
                const keyValuePairs = value.split(',').map(pair => pair.split(':'));
                const encodedPairs = keyValuePairs.map(([key, val]) => `${encodeURIComponent(key)}:${encodeURIComponent(val)}`);
                const keyValueString = encodedPairs.join(',');
                url.searchParams.set(name, keyValueString);
                queryParams[name] = keyValueString;
            }
        } else if (type === 'boolean' && value) {
            // Only add boolean parameters if they are true
            url.searchParams.set(name, true);
            queryParams[name] = true;
        } else if (type === 'integer' && !isNaN(parseInt(value, 10))) {
            url.searchParams.set(name, parseInt(value, 10));
            queryParams[name] = parseInt(value, 10);
        } else if (type === 'string' || type === 'themeName') {
            // Handle string and themeName types
            if (value) {
                url.searchParams.set(name, value);
                queryParams[name] = value;
            }
        } else if (type === 'abstime' && !isNaN(parseInt(value, 10))) {
            // Handle abstime (absolute time) parameters as duration offsets
            let timeValue = parseInt(value, 10);
            if (timeValue !== -1) {
                timeValue = Math.floor(Date.now() / 1000) + timeValue;
            }
            url.searchParams.set(name, timeValue);
            queryParams[name] = timeValue;
        }
    });

    // Retrieve encryption key and key ID from storage
    const b8KeyValue = await getStoredValue("b8KeyValue");
    const b8MgmtPassword = await getStoredValue("b8MgmtPassword");
    const b8KeyName = await getStoredValue("b8KeyName");
    const keyId = b8KeyName || 'default';
    const extLogging = await getStoredValue("extLogging");

    // Encrypt the query parameters
    let b64Json;
    if (b8KeyValue) {
        if(extLogging) {
            console.log("Encrypting locally with AES. " + JSON.stringify(queryParams));
        }
        b64Json = await encrypt(b8KeyValue, JSON.stringify(queryParams));
    } else {
        if(extLogging) {
            console.log("Encrypting via API.");
        }
        b64Json = await apiEncrypt(b8Host, keyId, queryParams, b8MgmtPassword);
    }

    // Construct the final encrypted URL
    const encryptedUrl = new URL(baseUrl);
    encryptedUrl.searchParams.set('keyId', keyId);
    encryptedUrl.searchParams.set('b8data', b64Json);

    return encryptedUrl;
}

export async function checkB8Host() {
    try {
        let b8CurrentlyEnabled = await getStoredValue("b8Enabled");
        // Get the stored value for b8Host
        let b8Host = await getStoredValue("b8Host");
        if (b8Host) {
            // Normalize the URL by removing trailing slashes
            b8Host = b8Host.replace(/\/+$/, '');
            let b8Url = new URL(b8Host);

            // Attempt to load viewer options from b8Host
            await loadViewerOptions(b8Host);
            const b8KeyValue = await getStoredValue("b8KeyValue");
            if(!b8KeyValue) {
                const b8MgmtPassword = await getStoredValue("b8MgmtPassword");
                const b8KeyName = await getStoredValue("b8KeyName");
                await apiEncrypt(b8Host, b8KeyName, {}, b8MgmtPassword);
            }

            // If successful, update the stored value
            if(!b8CurrentlyEnabled) {
                await setStoredValue("b8Enabled", true);
            }
        } else {
            console.warn("b8Host is not configured.");
            if(b8CurrentlyEnabled) {
                await setStoredValue("b8Enabled", false);
            }
        }
    } catch (e) {
        //console.error("Failed to load viewer options:", e.message);
        if(b8CurrentlyEnabled) {
            await setStoredValue("b8Enabled", false);
        }
    }
}
