/*
 * © 2024 Browsolate Limited. All rights reserved.
 *
 * This source code is the property of Browsolate Limited and is strictly confidential.
 * Unauthorized copying, modification, distribution, or use of this code, in part or in whole,
 * without prior written permission from Browsolate Limited is strictly prohibited and may
 * result in legal action.
 *
 * For licensing inquiries, please contact Browsolate Limited at https://browsolate.com.
 *
 * Author: Browsolate Limited
 * Version: 1.0
 * Date: 2024-11-02
 */

import {getStoredValue, loadViewerOptions} from "./common.js";

let ruleIdResetValue = -1;
let ruleIdCounter = 1; // Global counter for rule IDs

function generateUniqueRuleId() {
    if (ruleIdCounter > 2147483647) {
        ruleIdCounter = ruleIdResetValue; // Reset the counter if it exceeds the maximum allowed value
    }
    return ruleIdCounter++;
}
async function _addRulesForFileExtensions(listName, extensions, type, priority, extLogging) {
    if (extensions) {
        // Normalize extensions to ensure they are in "extension" format
        const normalizedExtensions = extensions.map(ext => {
            // Remove leading "*" if present
            if (ext.startsWith("*")) {
                ext = ext.substring(1);
            }
            // Remove leading "." if present
            if (ext.startsWith(".")) {
                ext = ext.substring(1);
            }
            return ext; // Return only the clean extension (e.g., "pdf", "js")
        });

        // Generate declarativeNetRequest rules for each extension
        const rules = normalizedExtensions.map((extension, index) => {
            return {
                id: generateUniqueRuleId(),
                priority: priority,
                action: { type: type },
                condition: {
                    regexFilter: `.*\\.${extension}($|\\?)`, // Match URLs ending in .extension
                    resourceTypes: ["main_frame"]
                }
            };
        });

        // Update declarativeNetRequest with the rules
        try {
            await updateDynamicRulesAsync({
                addRules: rules
            });
            if(extLogging) {
                console.log(`Added ${rules.length} ${type} rules for ${listName} extensions:`, normalizedExtensions);
            }
        } catch (error) {
            console.error(`Error adding ${type} rules for extensions:`, error.message);
        }
    }
}

function getDynamicRulesAsync() {
    return new Promise((resolve, reject) => {
        chrome.declarativeNetRequest.getDynamicRules((rules) => {
            if (chrome.runtime.lastError) {
                reject(new Error(chrome.runtime.lastError.message));
            } else {
                resolve(rules);
            }
        });
    });
}

function updateDynamicRulesAsync(rules) {
    return new Promise((resolve, reject) => {
        chrome.declarativeNetRequest.updateDynamicRules(rules, () => {
            if (chrome.runtime.lastError) {
                reject(new Error(chrome.runtime.lastError.message));
            } else {
                resolve();
            }
        });
    });
};

async function _addRulesForHosts(listName, domains, type, priority, extLogging) {
    if (domains) {
        // Generate declarativeNetRequest allow rules for each domain
        const rules = domains.map((domain) => ({
            id: generateUniqueRuleId(),
            priority: priority,
            action: { type: type },
            condition: {
                urlFilter: domain,
                resourceTypes: ["main_frame"]
            }
        }));

        // Wrap updateDynamicRules in a Promise to use async/await

        try {
            await updateDynamicRulesAsync({ addRules: rules });
            if (extLogging) {
                console.log(`Added ${rules.length} ${type} rules for ${listName} hosts:`, domains);
            }
        } catch (error) {
            console.error(`Error adding ${type} rules for ${listName}:`, error.message);
        }
    }
}
export async function addBlockRulesForFileExtensions(extLogging) {
    await _addRulesForFileExtensions("contentExtensions", await getStoredValue("contentExtensions"), "block", 1, extLogging);
}

export async function addBlockRulesForIncludedHosts(extLogging) {
    await _addRulesForHosts("includeDomains", await getStoredValue("includeDomains"), "block", 1, extLogging);
}

export async function addAllowRulesForExcludedHosts(extLogging) {
    await _addRulesForHosts("excludeDomains", await getStoredValue("excludeDomains"), "allow", 5000, extLogging);
}

export async function addTemporaryAllowRulesForExcludedHosts(extLogging) {
    let tempAllowList = [];
    await _addRulesForHosts("tempAllow", tempAllowList, "allow", 5000, extLogging);
}

export async function clearAllRules(extLogging) {
    try {
        ruleIdCounter = 1;
        const allowBrowsolateRuleId = generateUniqueRuleId();


        // Get all dynamic rules
        const dynamicRules = await getDynamicRulesAsync();
        const dynamicRuleIds = dynamicRules.map(rule => rule.id);

        if (dynamicRuleIds.length > 0) {
            await updateDynamicRulesAsync({
                removeRuleIds: dynamicRuleIds
            });
            if(extLogging) {
                console.log(`Cleared ${dynamicRuleIds.length} dynamic rules.`);
            }
        }

        let b8Enabled = await getStoredValue("b8Enabled");
        if(!b8Enabled) {
            if(await getStoredValue("failureMode") === "Open") {
                if(extLogging) {
                    console.log("FailOpen");
                }
                return;
            } else {
                if(extLogging) {
                    console.log("FailClosed");
                }
            }
        }

        let b8Host = await getStoredValue("b8Host");
        if (b8Host) {
            b8Host = b8Host.replace(/\/+$/, '');
            let b8Url = new URL(b8Host);

            await updateDynamicRulesAsync({
                addRules: [
                    {
                        id: allowBrowsolateRuleId,
                        priority: 2, // Higher priority
                        action: {type: "allow"},
                        condition: {
                            urlFilter: `|https://${b8Url.hostname}*`,
                            resourceTypes: ["main_frame"]
                        }
                    }
                ]
            });
        }
    } catch (error) {
        console.error("Error clearing rules and ensuring default rule:", error.message);
    }
    await addBlockRulesForIncludedHosts();
    await addBlockRulesForFileExtensions();
    await addAllowRulesForExcludedHosts();
    await addTemporaryAllowRulesForExcludedHosts();
    if(extLogging) {
        console.log(`${ruleIdCounter-1} Rules added:`);
        await logCurrentRules();
    }
    ruleIdResetValue = ruleIdCounter;
}

export async function logCurrentRules() {
    const rules1 = await getDynamicRulesAsync();
    console.log("Current Dynamic Rules:", rules1);
}

// Function to update the redirection rule in declarativeNetRequest with the dynamically generated encrypted URL
export async function updateRedirectionRule(targetUrl, encryptedUrl, extLogging) {
    // Fetch current dynamic rules
    const currentRules = await getDynamicRulesAsync();

    // Find a rule matching the targetUrl
    const matchingRule = currentRules.find(rule =>
        rule.condition?.urlFilter === targetUrl
    );

    // If a matching rule is found, remove it
    if (matchingRule) {
        //console.log(`Found existing rule for ${targetUrl}. Removing it.`);
        await updateDynamicRulesAsync({
            removeRuleIds: [matchingRule.id]
        });
    }

    // Generate a new unique ID for the rule
    const newRuleId = generateUniqueRuleId();


    // Add the new redirect rule pointing to the encrypted URL
    await updateDynamicRulesAsync({
        addRules: [
            {
                id: newRuleId,
                priority: 3,
                action: {
                    type: "redirect",
                    redirect: {
                        url: encryptedUrl
                    }
                },
                condition: {
                    urlFilter: targetUrl, // Match the specific target URL
                    resourceTypes: ["main_frame"]
                }
            }
        ]
    });

    if(extLogging) {
        console.log(`Added new dynamic rule (ID: ${newRuleId}) to redirect ${targetUrl} to ${encryptedUrl}.`);
    }
}
